@@ -3,6 +3,7 @@
 import pprint
 import random
 import re
+import tempfile
 from math import isclose, isnan
 
 import markdown
@@ -61,7 +62,9 @@ class BaseGrader(object):
         self.score = score
         self.submission = submission
 
-        if not isinstance(self.submission, type(self.answer)):
+        if not isinstance(
+            self.submission, type(self.answer)
+        ) and not isinstance(self.submission, tempfile._TemporaryFileWrapper):
             raise TypeError(
                 f"Your submission needs to be type {type(self.answer).__name__}, "
                 f"not type {type(self.submission).__name__}."
@@ -252,8 +255,14 @@ class PythonGrader(BaseGrader):
             return
 
         if match_order is False:
-            self.submission.sort()
-            self.answer.sort()
+            # For dealing with records
+            if isinstance(self.submission[0], dict):
+                sort_key = list(self.submission[0].keys())[0]
+                self.submission.sort(key=lambda x: x[sort_key])
+                self.answer.sort(key=lambda x: x[sort_key])
+            else:
+                self.submission.sort()
+                self.answer.sort()
 
         if not tolerance and self.submission == self.answer:
             self.add_to_score()
@@ -413,8 +422,9 @@ class PandasGrader(BaseGrader):
         Helper function used by ``grade_df`` and ``grade_series``.
         """
         message = str(message)
+        s = ""
 
-        if message.startswith("DataFrame"):
+        if "DataFrame" in message:
             if 'Attribute "names"' in message:
                 s = "The index name of your DataFrame doesn't"
             elif "index values" in message:
@@ -427,11 +437,13 @@ class PandasGrader(BaseGrader):
                 col = p.search(message).group(1)
                 s = f"The values in the `{col}` column in your DataFrame don't"
 
-        if message.startswith("Series.index"):
+        if "Series.index" in message:
             if 'Attribute "names"' in message:
                 s = "The index name of your Series doesn't"
             elif "index values" in message:
                 s = "The index values of your Series don't"
+            elif "dtype" in message:
+                s = "The dtype of your Series index doesn't"
 
         if message.startswith("Series are"):
             if 'Attribute "name"' in message:
@@ -521,7 +533,7 @@ class PandasGrader(BaseGrader):
         # Check shape
         if self.submission.shape != self.answer.shape:
             self.update_comment(
-                f"The shape of your DataFrame should be `{self.answer.shape}`,"
+                f"The shape of your {type(self.submission).__name__} should be `{self.answer.shape}`,"
                 f"not `{self.submission.shape}`."
             )
             if return_bool:
@@ -650,7 +662,7 @@ class PandasGrader(BaseGrader):
         # Check shape
         if self.submission.shape != self.answer.shape:
             self.update_comment(
-                f"The shape of your DataFrame should be `{self.answer.shape}`,"
+                f"The shape of your Series should be `{self.answer.shape}`,"
                 f"not `{self.submission.shape}`."
             )
             if return_bool:
@@ -673,6 +685,7 @@ class PandasGrader(BaseGrader):
             else:
                 return None
         except AssertionError as e:
+            print("Assertion Error", e)
             comment = self._clean_assert_message(e)
             self.update_comment(comment)
             if return_bool:
@@ -1228,6 +1241,8 @@ class PlotGrader(BaseGrader):
     """Grader for evaluating images."""
 
     def __init__(self, submission, answer, points=1):
+        if isinstance(submission, tempfile._TemporaryFileWrapper):
+            submission = Image.open(submission)
         super().__init__(submission, answer, points)
 
     def grade_plot_images(
